import { CubeTexture, CubeTextureLoader, Texture, TextureLoader } from 'three';
import { Asset, AssetType, AssetsEvents } from '../types/asset';
import { EventEmitter } from '../utils/events';
import { GLTF, GLTFLoader } from 'three/addons/loaders/GLTFLoader.js';
export declare class AssetManagerFactory extends EventEmitter<AssetsEvents> {
    assets: Asset[];
    texureLoader: TextureLoader;
    cubeTextureLoader: CubeTextureLoader;
    gltfLoader: GLTFLoader;
    constructor();
    /**
     * Get an asset by path, this is an unique identifier
     * @param path Asset path
     */
    getAssetByPath(path: string): Asset | undefined;
    /**
     * Create a new texture asset
     * @param data Data URI
     * @param name Asset name
     * @param type Texture type
     */
    createAsset(data: string, name: string, type?: AssetType): Promise<Asset>;
    /**
     * Load asset into a Texture or Model
     * @param asset Remote or local asset data
     */
    load(asset: Asset): Promise<Asset>;
    /**
     * Load assets into Textures
     * @param assets Remote or local asset data
     */
    loadAll(assets: Asset[]): Promise<void>;
    /**
     * Serialize for save file without Texture data
     */
    serialize(): Asset[];
    /**
     * Free all assets from memory
     */
    freeAll(): void;
    /**
     * Load texture
     * @param path Path
     * @param name Texture name
     */
    loadTextureData(path: string, name?: string): Promise<Texture>;
    /**
     * Load cube texture loader
     * @param path pos-x, neg-x, pos-y, neg-y, pos-z, neg-z
     * @param name Texture name
     */
    loadCubeTexture(path: string[], name?: string): Promise<CubeTexture>;
    loadMeshData(path: string, name?: string): Promise<GLTF>;
    loadJsonData(path: string): Promise<any>;
}
export declare const assetManager: AssetManagerFactory;
