import { CanvasTexture } from 'three';
export declare function rgbToHex(r: number, g: number, b: number): number;
type RadiusQuad = {
    tl: number;
    tr: number;
    br: number;
    bl: number;
};
export interface CanvasUtilsOptions {
    fill?: boolean;
    backgroundColor?: string;
    foregroundColor?: string;
    rounded?: boolean;
    roundedRadius?: number;
    textBorderColor?: string;
    textBorderSize?: number;
    textShadowBlur?: number;
}
export declare class CanvasUtils {
    private options?;
    constructor(options?: CanvasUtilsOptions | undefined);
    createTextCanvas(text: string | string[], bold?: boolean, fontSize?: number, padding?: number): {
        texture: CanvasTexture;
        width: number;
        height: number;
    };
    /**
     * Draws a rounded rectangle using the current state of the canvas.
     * If you omit the last three params, it will draw a rectangle
     * outline with a 5 pixel border radius
     *
     * https://stackoverflow.com/a/3368118
     *
     * @param {CanvasRenderingContext2D} ctx
     * @param {Number} x The top left x coordinate
     * @param {Number} y The top left y coordinate
     * @param {Number} width The width of the rectangle
     * @param {Number} height The height of the rectangle
     * @param {Number} [radius = 5] The corner radius; It can also be an object
     *                 to specify different radii for corners
     * @param {Number} [radius.tl = 0] Top left
     * @param {Number} [radius.tr = 0] Top right
     * @param {Number} [radius.br = 0] Bottom right
     * @param {Number} [radius.bl = 0] Bottom left
     * @param {Boolean} [fill = false] Whether to fill the rectangle.
     * @param {Boolean} [stroke = true] Whether to stroke the rectangle.
     */
    static roundRect(ctx: CanvasRenderingContext2D, x: number, y: number, width: number, height: number, radius?: number | RadiusQuad, fill?: boolean, stroke?: boolean): void;
    static readPixelDataRGB(image: HTMLImageElement): number[];
    static readPixelDataRScaled(image: HTMLImageElement, scale: number): number[];
}
export {};
