import { AmbientLight, Color, DirectionalLight } from 'three';
import { EngineComponent } from '../types/engine-component';
import { Environment } from '../gameobjects/environment.object';
import { environmentDefaults } from '../defaults/environment';
/**
 * This component manages game environment and world lighting.
 * @listens setEnvironment
 */
export class EnvironmentComponent extends EngineComponent {
    constructor(renderer, events) {
        super(renderer, events);
        this.renderer = renderer;
        this.events = events;
        this.name = EnvironmentComponent.name;
        this.object = new Environment();
    }
    initialize() {
        this.renderer.scene.add(this.object);
        this.renderer.renderer.setClearColor(environmentDefaults.clearColor);
        this.ambient = new AmbientLight(environmentDefaults.ambientColor, environmentDefaults.ambientStrength);
        this.directional = new DirectionalLight(environmentDefaults.sunColor, environmentDefaults.sunStrength);
        this.directional.position.copy(environmentDefaults.sunPosition);
        this.renderer.scene.add(this.ambient);
        this.renderer.scene.add(this.directional);
        this.cleanUpEvents = this.bindEvents();
    }
    update(delta) { }
    dispose() {
        this.renderer.scene.remove(this.ambient);
        this.renderer.scene.remove(this.directional);
        this.cleanUpEvents?.call(this);
    }
    bindEvents() {
        const setEnvironmentEvent = (event) => {
            if (event.sunColor)
                this.directional.color = new Color(event.sunColor);
            if (event.sunPosition)
                this.directional.position.copy(event.sunPosition);
            if (event.sunStrength)
                this.directional.intensity = event.sunStrength;
            if (event.ambientColor)
                this.ambient.color = new Color(event.ambientColor);
            if (event.ambientStrength)
                this.ambient.intensity = event.ambientStrength;
            if (event.clearColor)
                this.renderer.renderer.setClearColor(event.clearColor);
        };
        this.events.addListener('setEnvironment', setEnvironmentEvent);
        return () => {
            this.events.removeEventListener('setEnvironment', setEnvironmentEvent);
        };
    }
}
