import { Renderer } from '../core/renderer';
import { EngineComponent } from '../types/engine-component';
import { EngineEvents } from '../types/events';
import { EventEmitter } from '../utils/events';
import { Environment } from '../gameobjects/environment.object';
import { WorldFile } from '../types/world-file';
import { World } from '../gameobjects/world.object';
import { Object3D } from 'three';
import { GameObject, SerializedObject } from '../types/game-object';
/**
 * Game level management component. This component provides the World.
 *
 * EnvironmentComponent must be initialized before this component.
 * @listens change Applies changes to objects
 * @listens remove Removes objects from scene
 * @listens reparent Reparents object
 */
export declare class LevelComponent extends EngineComponent {
    protected renderer: Renderer;
    protected events: EventEmitter<EngineEvents>;
    name: string;
    world: World;
    private environment;
    private cleanUpEvents?;
    constructor(renderer: Renderer, events: EventEmitter<EngineEvents>);
    initialize(): void;
    update(delta: number): void;
    dispose(): void;
    getSceneTree(): {
        world: World;
        environment: Environment;
    };
    createObject(object: string, setParent?: Object3D, skipJoin?: boolean): GameObject | undefined;
    deserializeObject(root: SerializedObject, setParent?: Object3D): Object3D<import("three").Event> | undefined;
    serializeLevelSave(name: string): WorldFile;
    deserializeLevelSave(save: WorldFile): Promise<void>;
    private recursiveCreate;
    private bindEvents;
}
