import { Vector2 } from 'three';
import { EngineComponent } from '../types/engine-component';
/**
 * Manage viewport sizing
 */
export class ViewportComponent extends EngineComponent {
    constructor(render, events) {
        super(render, events);
        this.render = render;
        this.events = events;
        this.name = ViewportComponent.name;
    }
    get scene() {
        return this.render.scene;
    }
    get camera() {
        return this.render.camera;
    }
    initialize() {
        this.cleanUpEvents = this.bindEvents();
        this.camera.position.set(0, 4, 8);
    }
    update(dt) { }
    dispose() {
        this.cleanUpEvents?.call(this);
    }
    setSize(width, height) {
        this.render.viewport.style.width = `${width}px`;
        this.render.viewport.style.height = `${height}px`;
        this.render.setSize(width, height);
    }
    setSizeFromWindow() {
        this.events.emit('resize', new Vector2(window.innerWidth, window.innerHeight));
    }
    setSizeFromViewport() {
        this.events.emit('resize', new Vector2(this.render.viewport.parentElement.clientWidth, this.render.viewport.parentElement.clientHeight));
    }
    bindEvents() {
        const resizeEvent = (size) => {
            this.setSize(size.x, size.y);
        };
        this.events.addListener('resize', resizeEvent);
        return () => {
            this.events.removeEventListener('resize', resizeEvent);
        };
    }
}
