import { Camera, Mesh, Object3D, Quaternion, Raycaster, Vector3 } from 'three';
interface ExtendedPointer extends PointerEvent {
    x: number;
    y: number;
}
declare class DefinedProperties extends Object3D {
    [x: string]: any;
    camera: Camera;
    object?: Object3D;
    enabled: boolean;
    axis: string | null;
    mode: 'translate' | 'rotate' | 'scale';
    translationSnap: number;
    rotationSnap: number;
    scaleSnap: number;
    space: 'local' | 'world';
    size: number;
    dragging: boolean;
    showX: boolean;
    showY: boolean;
    showZ: boolean;
    worldPosition: Vector3;
    worldPositionStart: Vector3;
    worldQuaternion: Quaternion;
    worldQuaternionStart: Quaternion;
    cameraPosition: Vector3;
    cameraQuaternion: Quaternion;
    pointStart: Vector3;
    pointEnd: Vector3;
    rotationAxis: Vector3;
    rotationAngle: number;
    eye: Vector3;
}
/**
 * Transform controls - move, scale and rotate objects.
 */
declare class TransformControls extends DefinedProperties {
    camera: Camera;
    domElement: HTMLElement;
    isTransformControls: boolean;
    visible: boolean;
    private _gizmo;
    private _plane;
    private _offset;
    private _startNorm;
    private _endNorm;
    private _cameraScale;
    private _parentPosition;
    private _parentQuaternion;
    private _parentQuaternionInv;
    private _parentScale;
    private _worldScaleStart;
    private _worldQuaternionInv;
    private _worldScale;
    private _positionStart;
    private _quaternionStart;
    private _scaleStart;
    private _getPointer;
    private _onPointerDown;
    private _onPointerHover;
    private _onPointerMove;
    private _onPointerUp;
    constructor(camera: Camera, domElement: HTMLElement);
    updateMatrixWorld(): void;
    pointerHover(pointer: ExtendedPointer): void;
    pointerDown(pointer: PointerEvent): void;
    pointerMove(pointer: ExtendedPointer): void;
    pointerUp(pointer: ExtendedPointer): void;
    dispose(): void;
    attach(object: Object3D): this;
    detach(): this;
    reset(): void;
    getRaycaster(): Raycaster;
    getMode(): "scale" | "translate" | "rotate";
    setMode(mode: 'translate' | 'rotate' | 'scale'): void;
    setTranslationSnap(translationSnap: number): void;
    setRotationSnap(rotationSnap: number): void;
    setScaleSnap(scaleSnap: number): void;
    setSize(size: number): void;
    setSpace(space: 'world' | 'local'): void;
    private getPointer;
    private onPointerHover;
    private onPointerDown;
    private onPointerMove;
    private onPointerUp;
}
declare class TransformControlsGizmo extends DefinedProperties {
    isTransformControlsGizmo: boolean;
    type: string;
    gizmo: any;
    picker: any;
    helper: any;
    constructor();
    updateMatrixWorld(force?: boolean): void;
}
declare class TransformControlsPlane extends Mesh {
    [x: string]: any;
    isTransformControlsPlane: boolean;
    type: string;
    space: 'local' | 'world';
    worldPosition: Vector3;
    mode: string;
    worldQuaternion: Quaternion;
    axis: string | null;
    eye: Vector3;
    cameraQuaternion: Quaternion;
    constructor();
    updateMatrixWorld(force?: boolean): void;
}
export { TransformControls, TransformControlsGizmo, TransformControlsPlane };
