import { GameRunner } from '../types/game-runner';
import { Renderer } from './renderer';
export class Engine extends GameRunner {
    constructor() {
        super(...arguments);
        this.lastTick = performance.now();
        this.running = false;
        this.components = [];
    }
    mount(element) {
        this.element = element;
        this.render = new Renderer(element);
        this.render.renderer.autoClear = false;
    }
    loop(now) {
        throw new Error('Method not implemented.');
    }
    start() {
        this.running = true;
        this.loop(this.lastTick);
    }
    stop() {
        this.running = false;
        for (const component of this.components) {
            component.dispose();
        }
        this.render.dispose();
    }
    /**
     * Update every component.
     */
    update(dt) {
        for (const component of this.components) {
            component.update(dt);
        }
    }
    /**
     * Use and initialize an engine component
     * @param Component Engine Component
     */
    use(Component) {
        const component = new Component(this.render, this.events);
        this.components.push(component);
        component.initialize();
        return component;
    }
    /**
     * Get an engine component.
     * @param component Component
     */
    getComponent(component) {
        return this.components.find((cls) => cls.name === component.name);
    }
}
