import { BoxGeometry, CylinderGeometry, SphereGeometry } from 'three';
class GameObjectFactory {
    constructor() {
        this.boxGeometry = new BoxGeometry();
        this.sphereGeometry = new SphereGeometry(0.5);
        this.cylinderGeometry = new CylinderGeometry(0.5, 0.5);
        this.wedgeGeometry = new BoxGeometry();
        this.wedgeCornerGeometry = new BoxGeometry();
        this.wedgeInnerCornerGeometry = new BoxGeometry();
        this.makeProceduralShapes();
        this.finalize();
    }
    makeProceduralShapes() {
        const pos = this.wedgeGeometry.attributes.position;
        for (let i = 0; i < pos.count; i++) {
            if (pos.getX(i) < 0 && pos.getY(i) > 0)
                pos.setY(i, -0.5);
        }
        this.wedgeGeometry.computeVertexNormals();
        const pos2 = this.wedgeCornerGeometry.attributes.position;
        for (let i = 0; i < pos2.count; i++) {
            if (pos2.getY(i) > 0 && (pos2.getX(i) !== 0.5 || pos2.getZ(i) !== -0.5)) {
                pos2.setY(i, -0.5);
            }
        }
        this.wedgeCornerGeometry.computeVertexNormals();
        const pos3 = this.wedgeInnerCornerGeometry.attributes.position;
        for (let i = 0; i < pos3.count; i++) {
            if (pos3.getY(i) > 0 && pos3.getX(i) === 0.5 && pos3.getZ(i) === 0.5) {
                pos3.setY(i, -0.5);
            }
        }
        this.wedgeInnerCornerGeometry.computeVertexNormals();
    }
    finalize() {
        this.boxGeometry.computeBoundingBox();
        this.sphereGeometry.computeBoundingBox();
        this.cylinderGeometry.computeBoundingBox();
        this.wedgeGeometry.computeBoundingBox();
        this.wedgeCornerGeometry.computeBoundingBox();
        this.wedgeInnerCornerGeometry.computeBoundingBox();
    }
}
export const gameObjectFactory = new GameObjectFactory();
Object.freeze(gameObjectFactory);
