import { BoxGeometry, CapsuleGeometry, CylinderGeometry, SphereGeometry, TorusGeometry, } from 'three';
class WedgeGeometry extends BoxGeometry {
    constructor() {
        super();
        this.type = 'WedgeGeometry';
        const pos = this.attributes.position;
        for (let i = 0; i < pos.count; i++) {
            if (pos.getX(i) < 0 && pos.getY(i) > 0)
                pos.setY(i, -0.5);
        }
        this.computeVertexNormals();
    }
}
class WedgeCornerGeometry extends BoxGeometry {
    constructor() {
        super();
        this.type = 'WedgeCornerGeometry';
        const pos = this.attributes.position;
        for (let i = 0; i < pos.count; i++) {
            if (pos.getY(i) > 0 && (pos.getX(i) !== 0.5 || pos.getZ(i) !== -0.5)) {
                pos.setY(i, -0.5);
            }
        }
        this.computeVertexNormals();
    }
}
class WedgeInnerCornerGeometry extends BoxGeometry {
    constructor() {
        super();
        this.type = 'WedgeInnerCornerGeometry';
        const pos = this.attributes.position;
        for (let i = 0; i < pos.count; i++) {
            if (pos.getY(i) > 0 && pos.getX(i) === 0.5 && pos.getZ(i) === 0.5) {
                pos.setY(i, -0.5);
            }
        }
        this.computeVertexNormals();
    }
}
class GameObjectGeometryFactory {
    constructor() {
        this.boxGeometry = new BoxGeometry();
        this.sphereGeometry = new SphereGeometry(0.5);
        this.cylinderGeometry = new CylinderGeometry(0.5, 0.5);
        this.torusGeometry = new TorusGeometry();
        this.capsuleGeometry = new CapsuleGeometry(1, 1, 8, 16);
        this.wedgeGeometry = new WedgeGeometry();
        this.wedgeCornerGeometry = new WedgeCornerGeometry();
        this.wedgeInnerCornerGeometry = new WedgeInnerCornerGeometry();
        this.finalize();
    }
    finalize() {
        this.boxGeometry.computeBoundingBox();
        this.sphereGeometry.computeBoundingBox();
        this.cylinderGeometry.computeBoundingBox();
        this.torusGeometry.computeBoundingBox();
        this.capsuleGeometry.computeBoundingBox();
        this.wedgeGeometry.computeBoundingBox();
        this.wedgeCornerGeometry.computeBoundingBox();
        this.wedgeInnerCornerGeometry.computeBoundingBox();
    }
}
export const gameObjectGeometries = new GameObjectGeometryFactory();
Object.freeze(gameObjectGeometries);
