import { AnimationMixer, Skeleton, Vector3 } from 'three';
import { GameObject } from '../types/game-object';
import { MeshPart } from './mesh.object';
import { NameTag } from './nametag.object';
import { CanvasUtils } from '../canvas/utils';
import { PhysicsTicking } from '../physics';
import type Rapier from '@dimforge/rapier3d';
import { ServerTransformEvent } from '..';
export type HumanoidBodyPart = 'Head' | 'Torso' | 'ArmLeft' | 'ArmRight' | 'LegRight' | 'LegLeft';
export declare class Humanoid extends GameObject implements PhysicsTicking {
    isTickingObject: boolean;
    objectType: string;
    name: string;
    protected ready: boolean;
    protected skeleton: Skeleton;
    private _health;
    private _maxHealth;
    private _velocity;
    private _appliedGravity;
    private _adjustedPosition;
    private _grounded;
    private _lookAt;
    private _currentLookAt;
    private _animState;
    static bodyPartNames: HumanoidBodyPart[];
    static bodyBoneNames: string[];
    static bodyAnimationNames: string[];
    static nameTagBuilder: CanvasUtils;
    characterHeight: number;
    characterHalfHeight: number;
    protected shouldJump: boolean;
    protected mixer: AnimationMixer;
    private idleAction;
    private walkAction;
    private jumpAction;
    protected nameTag?: NameTag;
    protected collider?: Rapier.Collider;
    protected rigidBody?: Rapier.RigidBody;
    protected physicsWorldRef?: Rapier.World;
    protected characterControllerRef?: Rapier.KinematicCharacterController;
    mass: number;
    jumpPower: number;
    get health(): number;
    set health(value: number);
    get maxHealth(): number;
    set maxHealth(value: number);
    get alive(): boolean;
    get grounded(): boolean;
    set grounded(value: boolean);
    private get weight();
    private get bodyParts();
    getBodyPartByName(name: string): MeshPart;
    getBoneForBodyPart(part: HumanoidBodyPart): import("three").Bone | undefined;
    initialize(physicsEngine: typeof Rapier, physicsWorld: Rapier.World, characterController: Rapier.KinematicCharacterController): void;
    setVelocity(velocity: Vector3): void;
    setLook(vector: Vector3): void;
    jump(): void;
    tick(dt: number): void;
    setWalkAnimationState(index: number): void;
    detach(bodyPart?: HumanoidBodyPart): void;
    die(): void;
    synchronize(event: ServerTransformEvent): void;
    private createNameTag;
    private applyVelocity;
    private applyRotation;
    dispose(): void;
}
