var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { AnimationClip, AnimationMixer, LoopOnce, Vector3, } from 'three';
import { GameObject } from '../types/game-object';
import { EditorProperty } from '../decorators/property';
import { clamp } from 'three/src/math/MathUtils.js';
import { NameTag } from './nametag.object';
import { CanvasUtils } from '../canvas/utils';
class Humanoid extends GameObject {
    constructor() {
        super(...arguments);
        this.isTickingObject = true;
        this.objectType = 'Humanoid';
        this.name = 'Humanoid';
        this.ready = false;
        this._health = 100;
        this._maxHealth = 100;
        this._velocity = new Vector3(0, 0, 0);
        this._appliedGravity = new Vector3(0, 0, 0);
        this._adjustedPosition = new Vector3(0, 0, 0);
        this._grounded = true;
        this._lookAt = new Vector3(0, 0, 1);
        this._currentLookAt = new Vector3(0, 0, 1);
        this._animState = 0;
        this.characterHeight = 5.5;
        this.characterHalfHeight = this.characterHeight / 2;
        this.shouldJump = false;
        this.mass = -8;
        this.jumpPower = 8;
    }
    get health() {
        return this._health;
    }
    set health(value) {
        const health = clamp(Math.floor(value), 0, this.maxHealth);
        if (health === 0)
            this.die();
        this.health = health;
    }
    get maxHealth() {
        return this._maxHealth;
    }
    set maxHealth(value) {
        const maxHealth = Math.floor(value);
        if (this.health > maxHealth) {
            this.health = maxHealth;
        }
        this._maxHealth = maxHealth;
    }
    get alive() {
        return this.health > 0;
    }
    get grounded() {
        return this._grounded;
    }
    set grounded(value) {
        this._grounded = value;
    }
    get weight() {
        return (this.physicsWorldRef?.gravity.y || -9.81) + this.mass;
    }
    get bodyParts() {
        return Humanoid.bodyPartNames.map((key) => this.getBodyPartByName(key));
    }
    getBodyPartByName(name) {
        return this.parent?.getObjectByName(name);
    }
    getBoneForBodyPart(part) {
        return this.skeleton.getBoneByName(Humanoid.bodyBoneNames[Humanoid.bodyPartNames.indexOf(part)]);
    }
    initialize(physicsEngine, physicsWorld, characterController) {
        if (!this.parent)
            throw new Error('Cannot initialize Humanoid to empty parent');
        this.skeleton = (this.bodyParts[0]?.getMesh()).skeleton;
        if (!this.skeleton)
            throw new Error('Could not find Skeleton for Humanoid');
        this.skeleton.pose();
        // Clip actions
        this.mixer = new AnimationMixer(this.parent);
        this.ready = true;
        const idleClip = AnimationClip.findByName(this.parent.animations, 'Idle');
        const walkClip = AnimationClip.findByName(this.parent.animations, 'Walk');
        const jumpClip = AnimationClip.findByName(this.parent.animations, 'Jump');
        this.idleAction = this.mixer.clipAction(idleClip);
        this.walkAction = this.mixer.clipAction(walkClip);
        this.jumpAction = this.mixer.clipAction(jumpClip);
        this.jumpAction.setLoop(LoopOnce, 0);
        this.idleAction.play();
        // Create name tag
        this.createNameTag();
        if (this.virtual)
            return;
        this.physicsWorldRef = physicsWorld;
        this.characterControllerRef = characterController;
        // Character Physics
        const halfVec = new Vector3(0, this.characterHalfHeight, 0);
        const colliderDesc = physicsEngine.ColliderDesc.cuboid(1, this.characterHalfHeight, 0.5);
        const rigidBodyDesc = physicsEngine.RigidBodyDesc.kinematicPositionBased()
            .setTranslation(...this.parent.position.toArray())
            .setRotation(this.parent.quaternion);
        const rigidBody = physicsWorld.createRigidBody(rigidBodyDesc);
        const collider = physicsWorld.createCollider(colliderDesc, rigidBody);
        collider.setTranslationWrtParent(halfVec);
        this.rigidBody = rigidBody;
        this.collider = collider;
    }
    setVelocity(velocity) {
        this._velocity.copy(velocity);
        this.setWalkAnimationState(!!this._velocity.length() ? 1 : 0);
    }
    setLook(vector) {
        this._lookAt.lerp(vector, 0.15);
    }
    jump() {
        if (!this.grounded || this.shouldJump)
            return;
        this.shouldJump = true;
    }
    tick(dt) {
        if (!this.ready)
            return;
        // Stick to ground
        if (this.grounded) {
            this._appliedGravity.y = 0;
        }
        // Apply rigidbody transforms to object from last process tick
        if (this.rigidBody) {
            this.parent.position.copy(this.rigidBody.translation());
            this.parent.quaternion.copy(this.rigidBody.rotation());
        }
        // Run animation
        this.mixer.update(dt);
        // Apply jumping
        if (this.shouldJump) {
            this._appliedGravity.y = Math.sqrt(-2 * this.weight * this.jumpPower);
            this.grounded = false;
            this.shouldJump = false;
        }
        // Apply gravity
        this._appliedGravity.y += this.weight * dt;
        // Apply velocity
        this.applyVelocity(this._velocity.clone().add(this._appliedGravity).multiplyScalar(dt));
        // Apply look vector
        this._currentLookAt.copy(this.parent.position);
        this._currentLookAt.add(this._lookAt);
        this.parent?.lookAt(this._currentLookAt);
        this.applyRotation(this.parent.quaternion);
    }
    setWalkAnimationState(index) {
        const previousState = this._animState;
        this._animState = index;
        if (previousState === index)
            return;
        if (index === 1) {
            this.walkAction.reset().crossFadeFrom(this.idleAction, 0.5, false).play();
        }
        else {
            this.walkAction.crossFadeTo(this.idleAction.reset(), 0.5, false).play();
        }
    }
    detach(bodyPart) {
        if (!bodyPart) {
            Humanoid.bodyPartNames.forEach((part) => this.detach(part));
            return;
        }
        const part = this.getBodyPartByName(bodyPart);
        if (!part)
            return;
        const partMesh = part.getMesh();
        if (partMesh.bindMode === 'detached')
            return;
        partMesh.bindMode = 'detached';
        partMesh.updateMatrixWorld(true);
        const bone = this.getBoneForBodyPart(bodyPart);
        if (!bone)
            return;
        bone.removeFromParent();
        this.skeleton.update();
    }
    die() {
        if (!this.alive)
            return;
        this.health = 0;
        this.detach();
    }
    synchronize(event) {
        if (!this.rigidBody)
            return;
        // this.parent!.position.copy(this.rigidBody.translation() as any);
        // this.parent!.quaternion.copy(this.rigidBody.rotation() as any);
        this.rigidBody?.setNextKinematicTranslation(event.position);
        this.rigidBody?.setRotation(event.quaternion, false);
    }
    createNameTag() {
        if (this.nameTag) {
            this.nameTag.dispose();
        }
        this.nameTag = NameTag.create(Humanoid.nameTagBuilder, this.parent.name);
        this.nameTag.position.set(0, 1.5, 0);
        this.add(this.nameTag);
    }
    applyVelocity(velocity) {
        if (!this.characterControllerRef || !this.parent || !this.rigidBody)
            return;
        const vec3 = this.parent.position.clone();
        this.characterControllerRef.computeColliderMovement(this.collider, velocity);
        const computed = this.characterControllerRef.computedMovement();
        const grounded = this.characterControllerRef.computedGrounded();
        vec3.copy(computed);
        this.rigidBody?.setNextKinematicTranslation(vec3.add(this.parent.position));
        // After the collider movement calculation is done, we can read the
        // collision events.
        // for (let i = 0; i < this.controller.numComputedCollisions(); i++) {
        //   let collision = this.controller.computedCollision(i);
        //   // Do something with that collision information.
        //   console.log(collision);
        // }
        this._grounded = grounded;
    }
    applyRotation(quat) {
        this.rigidBody?.setRotation(quat, false);
    }
    dispose() {
        this.nameTag?.dispose();
        if (this.collider && !this.rigidBody) {
            this.physicsWorldRef?.removeCollider(this.collider, false);
        }
        if (this.rigidBody) {
            this.physicsWorldRef?.removeRigidBody(this.rigidBody);
        }
    }
}
Humanoid.bodyPartNames = [
    'Head',
    'Torso',
    'ArmLeft',
    'ArmRight',
    'LegRight',
    'LegLeft',
];
Humanoid.bodyBoneNames = [
    'BHead',
    'BTorso',
    'BArmL',
    'BArmR',
    'BLegR',
    'BLegL',
];
Humanoid.bodyAnimationNames = ['Idle', 'Walk'];
Humanoid.nameTagBuilder = new CanvasUtils({
    fill: false,
    textBorderColor: '#000',
    foregroundColor: '#fff',
    textShadowBlur: 2,
    textBorderSize: 1,
});
__decorate([
    EditorProperty({ type: Number }),
    __metadata("design:type", Object)
], Humanoid.prototype, "mass", void 0);
__decorate([
    EditorProperty({ type: Boolean }),
    __metadata("design:type", Object)
], Humanoid.prototype, "jumpPower", void 0);
__decorate([
    EditorProperty({ type: Number }),
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [Number])
], Humanoid.prototype, "health", null);
__decorate([
    EditorProperty({ type: Number }),
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [Number])
], Humanoid.prototype, "maxHealth", null);
export { Humanoid };
