var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { PhysicalObject } from './physical.object';
import { Quaternion, Vector3 } from 'three';
import { EditorProperty } from '../decorators/property';
export class PhysicsObject extends PhysicalObject {
    constructor() {
        super(...arguments);
        this.objectType = 'PhysicsObject';
        this.isTickingObject = true;
        this.canCollide = true;
        this.anchored = true;
        this.mass = 1;
        this.friction = 0.2;
        this._tempVec = new Vector3();
        this._tempQuat = new Quaternion();
        this._tempQuat2 = new Quaternion();
    }
    initialize(physicsEngine, physicsWorld) {
        if (this.virtual)
            return;
        this.physicsWorldRef = physicsWorld;
        // This object has effecively no physics:
        // doesn't move, doesn't collide
        if (this.anchored && !this.canCollide)
            return;
        let bodyDesc;
        if (this.anchored)
            bodyDesc = physicsEngine.RigidBodyDesc.fixed();
        else
            bodyDesc = physicsEngine.RigidBodyDesc.dynamic();
        const position = this.position.clone();
        const rotation = this.quaternion.clone();
        this.getWorldPosition(position);
        this.getWorldQuaternion(rotation);
        bodyDesc
            .setTranslation(...position.toArray())
            .setRotation(rotation)
            .setAdditionalMass(this.mass)
            .setLinearDamping(this.friction)
            .setAngularDamping(this.friction);
        const body = physicsWorld.createRigidBody(bodyDesc);
        let collider;
        if (this.canCollide) {
            collider = this.createCollider(physicsEngine, physicsWorld, body);
        }
        this.collider = collider;
        this.rigidBody = body;
    }
    tick(dt) {
        if (!this.rigidBody || this.virtual)
            return;
        this._tempVec.copy(this.rigidBody.translation());
        this._tempQuat.copy(this.rigidBody.rotation());
        if (this.parent?.objectType !== 'World') {
            this.parent?.worldToLocal(this._tempVec);
            this.parent?.getWorldQuaternion(this._tempQuat2);
            this._tempQuat2.invert();
            this._tempQuat.premultiply(this._tempQuat2);
        }
        this.position.copy(this._tempVec);
        this.quaternion.copy(this._tempQuat);
    }
    dispose() {
        if (this.collider && !this.rigidBody) {
            this.physicsWorldRef?.removeCollider(this.collider, false);
        }
        if (this.rigidBody) {
            this.physicsWorldRef?.removeRigidBody(this.rigidBody);
        }
    }
    /**
     * Create collision shape for physics object.
     * @internal
     * @param factory Physics engine
     * @param world Physics world
     * @param body RigidBody
     */
    createCollider(factory, world, body) {
        return undefined;
    }
}
__decorate([
    EditorProperty({ type: Boolean }),
    __metadata("design:type", Object)
], PhysicsObject.prototype, "canCollide", void 0);
__decorate([
    EditorProperty({ type: Boolean }),
    __metadata("design:type", Object)
], PhysicsObject.prototype, "anchored", void 0);
__decorate([
    EditorProperty({ type: Number }),
    __metadata("design:type", Object)
], PhysicsObject.prototype, "mass", void 0);
__decorate([
    EditorProperty({ type: Number }),
    __metadata("design:type", Object)
], PhysicsObject.prototype, "friction", void 0);
