import { SmartBuffer } from 'smart-buffer';
import { Quaternion, Vector3 } from 'three';
export class Packet {
    constructor(packet) {
        this.packet = packet;
        this.buffer = new SmartBuffer();
    }
    write(data, type) {
        switch (type) {
            case 'string':
            case String:
                this.buffer.writeStringNT(data);
                break;
            case 'bool':
            case Boolean:
                this.buffer.writeUInt8(data ? 1 : 0);
                break;
            case 'float':
            case Number:
                this.buffer.writeFloatLE(data);
                break;
            case 'uint8':
                this.buffer.writeUInt8(data);
                break;
            case 'int32':
                this.buffer.writeInt32LE(data);
                break;
            case 'uint32':
                this.buffer.writeUInt32LE(data);
                break;
            case 'vec3':
                this.buffer.writeFloatLE(data.x);
                this.buffer.writeFloatLE(data.y);
                this.buffer.writeFloatLE(data.z);
                break;
            case 'quat':
                this.buffer.writeFloatLE(data.x);
                this.buffer.writeFloatLE(data.y);
                this.buffer.writeFloatLE(data.z);
                this.buffer.writeFloatLE(data.w);
                break;
        }
        return this;
    }
    read(type) {
        switch (type) {
            case 'string':
            case String:
                return this.buffer.readStringNT();
            case 'bool':
            case Boolean:
                return this.buffer.readUInt8();
            case 'float':
            case Number:
                return this.buffer.readFloatLE();
            case 'uint8':
                return this.buffer.readUInt8();
            case 'int32':
                return this.buffer.readInt32LE();
            case 'uint32':
                return this.buffer.readUInt32LE();
            case 'vec3':
                return new Vector3(this.buffer.readFloatLE(), this.buffer.readFloatLE(), this.buffer.readFloatLE());
            case 'quat':
                return new Quaternion(this.buffer.readFloatLE(), this.buffer.readFloatLE(), this.buffer.readFloatLE(), this.buffer.readFloatLE());
        }
    }
    writeHeader() {
        if (this.packet === undefined)
            return;
        this.buffer.insertUInt8(this.packet, 0);
    }
    toBuffer() {
        this.writeHeader();
        return this.buffer.toBuffer();
    }
    static from(buffer) {
        const packet = new Packet();
        packet.buffer = SmartBuffer.fromBuffer(buffer);
        packet.packet = packet.buffer.readUInt8();
        return packet;
    }
}
