import { Brick, Cylinder, Sphere, Torus, Capsule, Wedge, WedgeCorner, WedgeInnerCorner, } from '../gameobjects';
import { Matrix4 } from 'three';
class ColliderFactory {
    static createCollider(obj, factory, world, body) {
        const collider = ColliderFactory.gameObjectColliders.find((entry) => obj instanceof entry.instance);
        if (!collider)
            return undefined;
        return collider.createCollisionShape(obj, factory, world, body);
    }
}
ColliderFactory.gameObjectColliders = [
    {
        instance: Cylinder,
        createCollisionShape: (obj, factory, world, body) => {
            const height = obj.scale.y / 2;
            const radius = (obj.scale.x / 2 + obj.scale.z / 2) / 2;
            const collider = factory.ColliderDesc.cylinder(height, radius);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: Sphere,
        createCollisionShape: (obj, factory, world, body) => {
            const radius = Math.max(obj.scale.x, obj.scale.y, obj.scale.z) / 2;
            const collider = factory.ColliderDesc.ball(radius);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: Torus,
        createCollisionShape: (obj, factory, world, body) => {
            const height = obj.scale.y / 2;
            const radius = (obj.scale.x / 2 + obj.scale.z / 2) / 2;
            const collider = factory.ColliderDesc.roundCylinder(height, radius, height);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: Capsule,
        createCollisionShape: (obj, factory, world, body) => {
            const height = obj.scale.y / 2;
            const radius = (obj.scale.x / 2 + obj.scale.z / 2) / 2;
            const collider = factory.ColliderDesc.capsule(height, radius);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: Wedge,
        createCollisionShape: (obj, factory, world, body) => {
            const mat = new Matrix4();
            mat.makeScale(...obj.scale.toArray());
            const points = obj
                .getGeometry()
                .getAttribute('position')
                .clone()
                .applyMatrix4(mat)?.array;
            const collider = factory.ColliderDesc.convexMesh(points);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: WedgeCorner,
        createCollisionShape: (obj, factory, world, body) => {
            const mat = new Matrix4();
            mat.makeScale(...obj.scale.toArray());
            const points = obj
                .getGeometry()
                .getAttribute('position')
                .clone()
                .applyMatrix4(mat)?.array;
            const collider = factory.ColliderDesc.convexMesh(points);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: WedgeInnerCorner,
        createCollisionShape: (obj, factory, world, body) => {
            const mat = new Matrix4();
            mat.makeScale(...obj.scale.toArray());
            const points = obj
                .getGeometry()
                .getAttribute('position')
                .clone()
                .applyMatrix4(mat)?.array;
            const collider = factory.ColliderDesc.convexMesh(points);
            return world.createCollider(collider, body);
        },
    },
    {
        instance: Brick,
        createCollisionShape: (obj, factory, world, body) => {
            const scale = obj.scale.clone().divideScalar(2);
            const collider = factory.ColliderDesc.cuboid(scale.x, scale.y, scale.z);
            return world.createCollider(collider, body);
        },
    },
];
export { ColliderFactory };
