export class PhysicsObjectAssociation {
    constructor(object, collider, body) {
        this.object = object;
        this.collider = collider;
        this.body = body;
        this.isTickingObject = true;
        this.uuid = object.uuid;
    }
    initialize() { }
    tick(dt) {
        if (!this.body)
            return;
        this.object.position.copy(this.body.translation());
        this.object.quaternion.copy(this.body.rotation());
    }
}
export class HumanoidPhysicsProxy extends PhysicsObjectAssociation {
    constructor(humanoid, controller, collider, body) {
        super(humanoid.parent, collider, body);
        this.humanoid = humanoid;
        this.controller = controller;
    }
    applyMovement(position, velocity) {
        const vec3 = position.clone();
        this.controller.computeColliderMovement(this.collider, velocity);
        const computed = this.controller.computedMovement();
        const grounded = this.controller.computedGrounded();
        vec3.copy(computed);
        this.body?.setNextKinematicTranslation(vec3.add(position));
        // After the collider movement calculation is done, we can read the
        // collision events.
        // for (let i = 0; i < this.controller.numComputedCollisions(); i++) {
        //   let collision = this.controller.computedCollision(i);
        //   // Do something with that collision information.
        //   console.log(collision);
        // }
        // this.collider?.setTranslation(colliderHalf);
        // this.humanoid.parent!.position.copy(computed as Vector3).sub(halfVec);
        this.humanoid.grounded = grounded;
    }
    applyRotation(quat) {
        this.body?.setNextKinematicRotation(quat);
    }
}
