import { Euler, Object3D, Vector3 } from 'three';
import { Property } from './property';
export declare class GameObject extends Object3D {
    /**
     * Type of this Game Object.
     */
    objectType: string;
    /**
     * Virtual game objects are usually built-in types that
     * do not have physical representations in the game.
     *
     * These types still extend Object3D so they could
     * have effects in the 3D world.
     */
    virtual: boolean;
    uuid: string;
    name: string;
    visible: boolean;
    /**
     * Object gets included in exported files if `true`.
     * @default true
     */
    archivable: boolean;
    constructor();
    private get excludedProperties();
    /**
     * The exposed properties for this game object, used for the editor
     */
    get properties(): Property[];
    /**
     * Serialize GameObject for exporting
     */
    serialize(): SerializedObject;
    copy(object: Object3D, recursive?: boolean): this;
    /**
     * This function can be used to modify any property of the object by key.
     * @param key Key to set
     * @param value Value to set
     */
    setOwnProperty(key: keyof GameObject, value: unknown): void;
    setOwnProperty(key: string, value: unknown): void;
    /**
     * Deserialize a serialized object into properties on this object.
     * @param input Serialized information
     */
    deserialize(input: SerializedObject): void;
}
export declare class GameObject3D extends GameObject {
    objectType: string;
    locked: boolean;
    position: Vector3;
    scale: Vector3;
    rotation: Euler;
}
export interface SerializedObject {
    [x: string]: unknown;
    name: string;
    objectType: string;
    children: SerializedObject[];
    visible: boolean;
}
