var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Color, Euler, Object3D, Vector3 } from 'three';
import { EditorProperty } from '../decorators/property';
import { readMetadataOf } from '../utils/read-metadata';
export class GameObject extends Object3D {
    constructor() {
        super();
        /**
         * Type of this Game Object.
         */
        this.objectType = 'GameObject';
        /**
         * Virtual game objects are usually built-in types that
         * do not have physical representations in the game.
         *
         * These types still extend Object3D so they could
         * have effects in the 3D world.
         */
        this.virtual = false;
        this.name = '';
        this.visible = true;
        /**
         * Object gets included in exported files if `true`.
         * @default true
         */
        this.archivable = true;
        this.name = this.objectType;
    }
    get excludedProperties() {
        return readMetadataOf(this, 'excludedProperties');
    }
    /**
     * The exposed properties for this game object, used for the editor
     */
    get properties() {
        const exclude = this.excludedProperties;
        const properties = readMetadataOf(this, 'properties');
        return properties.filter((item) => !exclude.includes(item.name));
    }
    /**
     * Serialize GameObject for exporting
     */
    serialize() {
        const object = {
            name: this.name,
            objectType: this.objectType,
            children: this.children
                .filter((entry) => entry instanceof GameObject && entry.archivable !== false)
                .map((entry) => entry.serialize()),
            visible: this.visible,
        };
        const keys = this.properties.map((property) => property.name);
        Object.assign(object, keys.reduce((obj, key) => {
            const indexable = this;
            const value = indexable[key]?.toArray
                ? indexable[key].toArray()
                : indexable[key];
            return {
                ...obj,
                [key]: value,
            };
        }, {}));
        return object;
    }
    copy(object, recursive = true) {
        super.copy(object, recursive);
        this.properties
            .map((property) => property.name)
            .filter((key) => !['position', 'rotation', 'scale', 'uuid'].includes(key))
            .forEach((key) => this.setOwnProperty(key, object[key]));
        return this;
    }
    setOwnProperty(key, value) {
        const indexable = this;
        if (indexable[key]?.fromArray && Array.isArray(value)) {
            indexable[key].fromArray(value);
        }
        else if (indexable[key]?.isColor) {
            indexable[key] = new Color(value);
        }
        else if (indexable[key]?.copy) {
            indexable[key].copy(value);
        }
        else {
            indexable[key] = value;
        }
    }
    /**
     * Deserialize a serialized object into properties on this object.
     * @param input Serialized information
     */
    deserialize(input) {
        Object.keys(input)
            .filter((key) => !['children', 'objectType'].includes(key))
            .forEach((key) => this.setOwnProperty(key, input[key]));
    }
}
__decorate([
    EditorProperty({ type: String, exposed: false, readonly: true }),
    __metadata("design:type", String)
], GameObject.prototype, "uuid", void 0);
__decorate([
    EditorProperty({ type: String }),
    __metadata("design:type", String)
], GameObject.prototype, "name", void 0);
__decorate([
    EditorProperty({ type: Boolean }),
    __metadata("design:type", Boolean)
], GameObject.prototype, "visible", void 0);
__decorate([
    EditorProperty({ type: Boolean }),
    __metadata("design:type", Boolean)
], GameObject.prototype, "archivable", void 0);
export class GameObject3D extends GameObject {
    constructor() {
        super(...arguments);
        this.objectType = 'GameObject3D';
        this.locked = false;
    }
}
__decorate([
    EditorProperty({ type: Boolean }),
    __metadata("design:type", Object)
], GameObject3D.prototype, "locked", void 0);
__decorate([
    EditorProperty({ type: Vector3 }),
    __metadata("design:type", Vector3)
], GameObject3D.prototype, "position", void 0);
__decorate([
    EditorProperty({ type: Vector3 }),
    __metadata("design:type", Vector3)
], GameObject3D.prototype, "scale", void 0);
__decorate([
    EditorProperty({ type: Euler }),
    __metadata("design:type", Euler)
], GameObject3D.prototype, "rotation", void 0);
