export class EventEmitter {
    constructor() {
        this._handlers = {};
    }
    addListener(event, fn, once = false) {
        if (typeof fn !== 'function') {
            return;
        }
        if (!this._handlers[event]) {
            this._handlers[event] = [];
        }
        this._handlers[event].push({ fn, once });
    }
    on(event, fn) {
        this.addListener(event, fn, false);
    }
    once(event, fn) {
        this.addListener(event, fn, true);
    }
    emit(event, ...args) {
        if (!this._handlers[event]) {
            return;
        }
        this._handlers[event]
            .filter((handler) => handler && typeof handler.fn === 'function')
            .forEach((handler) => {
            handler.fn(...args);
            if (handler.once) {
                this.removeEventListener(event, handler.fn);
            }
        });
    }
    removeEventListener(event, fn) {
        if (!this._handlers[event] || typeof fn !== 'function') {
            return;
        }
        const indexOf = this._handlers[event].findIndex((entry) => entry.fn === fn);
        if (indexOf > -1) {
            this._handlers[event].splice(indexOf, 1);
        }
    }
    removeAllListeners(event) {
        if (!this._handlers[event]) {
            return;
        }
        delete this._handlers[event];
    }
}
