import { Object3D, Vector2 } from 'three';
import { EngineEvents } from '../types/events';
import { EngineComponent } from '../types/engine-component';
import { Renderer } from '../core/renderer';
import { EventEmitter } from '../utils/events';

/**
 * Manage viewport sizing
 */
export class ViewportComponent extends EngineComponent {
  public name = ViewportComponent.name;
  private cleanUpEvents?: Function;

  constructor(
    protected render: Renderer,
    protected events: EventEmitter<EngineEvents>
  ) {
    super(render, events);
  }

  get scene() {
    return this.render.scene;
  }

  get camera() {
    return this.render.camera;
  }

  initialize() {
    this.cleanUpEvents = this.bindEvents();
    this.camera.position.set(0, 4, 8);
  }

  update(dt: number) {}

  dispose(): void {
    this.cleanUpEvents?.call(this);
  }

  setSize(width: number, height: number) {
    this.render.viewport.style.width = `${width}px`;
    this.render.viewport.style.height = `${height}px`;
    this.render.setSize(width, height);
  }

  setSizeFromWindow() {
    this.events.emit(
      'resize',
      new Vector2(window.innerWidth, window.innerHeight)
    );
  }

  setSizeFromViewport() {
    this.events.emit(
      'resize',
      new Vector2(
        this.render.viewport.parentElement!.clientWidth,
        this.render.viewport.parentElement!.clientHeight
      )
    );
  }

  private bindEvents() {
    const resizeEvent = (size: Vector2) => {
      this.setSize(size.x, size.y);
    };

    this.events.addListener('resize', resizeEvent);

    return () => {
      this.events.removeEventListener('resize', resizeEvent);
    };
  }
}
