import {
  BoxGeometry,
  CapsuleGeometry,
  CylinderGeometry,
  SphereGeometry,
  TorusGeometry,
} from 'three';

class WedgeGeometry extends BoxGeometry {
  type = 'WedgeGeometry';

  constructor() {
    super();
    const pos = this.attributes.position;
    for (let i = 0; i < pos.count; i++) {
      if (pos.getX(i) < 0 && pos.getY(i) > 0) pos.setY(i, -0.5);
    }
    this.computeVertexNormals();
  }
}

class WedgeCornerGeometry extends BoxGeometry {
  type = 'WedgeCornerGeometry';

  constructor() {
    super();
    const pos = this.attributes.position;
    for (let i = 0; i < pos.count; i++) {
      if (pos.getY(i) > 0 && (pos.getX(i) !== 0.5 || pos.getZ(i) !== -0.5)) {
        pos.setY(i, -0.5);
      }
    }
    this.computeVertexNormals();
  }
}

class WedgeInnerCornerGeometry extends BoxGeometry {
  type = 'WedgeInnerCornerGeometry';
  constructor() {
    super();
    const pos = this.attributes.position;
    for (let i = 0; i < pos.count; i++) {
      if (pos.getY(i) > 0 && pos.getX(i) === 0.5 && pos.getZ(i) === 0.5) {
        pos.setY(i, -0.5);
      }
    }
    this.computeVertexNormals();
  }
}

class GameObjectGeometryFactory {
  public boxGeometry = new BoxGeometry();
  public sphereGeometry = new SphereGeometry(0.5);
  public cylinderGeometry = new CylinderGeometry(0.5, 0.5);
  public torusGeometry = new TorusGeometry();
  public capsuleGeometry = new CapsuleGeometry(1, 1, 8, 16);
  public wedgeGeometry = new WedgeGeometry();
  public wedgeCornerGeometry = new WedgeCornerGeometry();
  public wedgeInnerCornerGeometry = new WedgeInnerCornerGeometry();

  constructor() {
    this.finalize();
  }

  private finalize() {
    this.boxGeometry.computeBoundingBox();
    this.sphereGeometry.computeBoundingBox();
    this.cylinderGeometry.computeBoundingBox();
    this.torusGeometry.computeBoundingBox();
    this.capsuleGeometry.computeBoundingBox();
    this.wedgeGeometry.computeBoundingBox();
    this.wedgeCornerGeometry.computeBoundingBox();
    this.wedgeInnerCornerGeometry.computeBoundingBox();
  }
}

export const gameObjectGeometries = new GameObjectGeometryFactory();
Object.freeze(gameObjectGeometries);
