import { Sprite, CanvasTexture, SpriteMaterial } from 'three';
import { CanvasUtils } from '../canvas';
import { Disposable } from '../types/disposable';
import { GameObject } from '../types/game-object';

export class NameTag extends GameObject implements Disposable {
  public objectType = 'NameTag';
  public virtual = true;
  public archivable = false;

  public width!: number;
  public height!: number;
  private texture!: CanvasTexture;
  private material!: SpriteMaterial;

  public static create(builder: CanvasUtils, name: string) {
    const { texture, width, height } = builder.createTextCanvas(
      name,
      false,
      48
    );
    const obj = new NameTag();

    obj.texture = texture;
    obj.width = width;
    obj.height = height;

    obj.material = new SpriteMaterial({
      map: texture,
      transparent: true,
    });

    const label = new Sprite(obj.material);

    const labelBaseScale = 0.01;
    label.scale.x = width * labelBaseScale;
    label.scale.y = height * labelBaseScale;

    obj.add(label);

    return obj;
  }

  dispose() {
    this.material.dispose();
    this.texture.dispose();
  }
}
