import { GameObject3D } from '../types/game-object';
import {
  BufferGeometry,
  Color,
  ColorRepresentation,
  Mesh,
  MeshPhongMaterial,
} from 'three';
import { assetManager } from '../assets/manager';
import { AssetInfo } from '../types/asset';
import { EditorProperty } from '../decorators/property';

export class PhysicalObject extends GameObject3D {
  private texturePath?: string;

  @EditorProperty({ type: Color })
  get color() {
    return this.material.color;
  }
  set color(color: ColorRepresentation) {
    this.material.color = new Color(color);
  }

  @EditorProperty({ type: Number })
  get transparency() {
    return 1 - this.material.opacity;
  }
  set transparency(value: number) {
    this.material.transparent = value != 0;
    this.material.opacity = 1 - value;
    this.material.needsUpdate = true;
  }

  @EditorProperty({ type: AssetInfo })
  get texture() {
    return this.texturePath;
  }
  set texture(path: string | undefined) {
    if (!path) {
      this.material.map = null;
      this.texturePath = undefined;
      this.material.needsUpdate = true;
      return;
    }

    const asset = assetManager.getAssetByPath(path);
    if (!asset || !asset.texture) {
      console.error(`Asset ${path} does not exist or is not loaded`);
      return;
    }

    this.texturePath = path;
    this.material.map = asset.texture;
    this.material.needsUpdate = true;
  }

  constructor(
    protected geometry: BufferGeometry,
    protected material = new MeshPhongMaterial(),
    protected mesh: Mesh = new Mesh(geometry, material)
  ) {
    super();
    this.name = this.objectType;
    this.add(this.mesh);
  }

  getGeometry() {
    return this.geometry;
  }

  getMesh() {
    return this.mesh;
  }
}
