import { Mesh, Quaternion, Vector3 } from 'three';
import { Brick } from './brick.object';
import { gameObjectGeometries } from './geometries';
import type Rapier from '@dimforge/rapier3d';

export class Torus extends Brick {
  public objectType = 'Torus';
  protected mesh = new Mesh(this.geometry, this.material);

  constructor() {
    super(gameObjectGeometries.torusGeometry);
    this.name = this.objectType;
  }

  protected override createCollider(
    factory: typeof Rapier,
    world: Rapier.World,
    body?: Rapier.RigidBody
  ) {
    const scale = this.getWorldScale(new Vector3());
    const height = Math.abs(scale.y) * 0.4;
    const radius = (Math.abs(scale.x) + Math.abs(scale.z)) / 2;
    const collider = factory.ColliderDesc.cylinder(
      height,
      radius * 1.4
    ).setRotation(
      new Quaternion().setFromAxisAngle(new Vector3(1, 0, 0), Math.PI / 2)
    );
    return world.createCollider(collider, body);
  }
}
