import { Matrix4, Mesh, Vector3 } from 'three';
import { Brick } from './brick.object';
import { gameObjectGeometries } from './geometries';
import type Rapier from '@dimforge/rapier3d';

export class Wedge extends Brick {
  public objectType = 'Wedge';
  protected mesh = new Mesh(this.geometry, this.material);

  constructor() {
    super(gameObjectGeometries.wedgeGeometry);
    this.name = this.objectType;
  }

  protected override createCollider(
    factory: typeof Rapier,
    world: Rapier.World,
    body?: Rapier.RigidBody
  ) {
    const scale = this.getWorldScale(new Vector3());
    const mat = new Matrix4();
    mat.makeScale(...scale.toArray());
    const points = this.getGeometry()
      .getAttribute('position')
      .clone()
      .applyMatrix4(mat)?.array as Float32Array;
    const collider = factory.ColliderDesc.convexMesh(points)!;
    return world.createCollider(collider, body);
  }
}
