export abstract class GameRunner {
  /**
   * Last tick length in milliseconds.
   */
  public lastTick = performance.now();

  /**
   * Game running flag.
   */
  public running = false;

  /**
   * Create the viewport and start the game.
   * @param element Viewport element
   */
  abstract mount(element: HTMLElement): void;

  /**
   * This is called on every frame of the game.
   * Use this to render and update your components.
   */
  abstract loop(now: DOMHighResTimeStamp): void;

  /**
   * Start running.
   */
  abstract start(): void;

  /**
   * Stop running.
   */
  abstract stop(): void;

  /**
   * Get delta time in seconds.
   * @param now Time since last frame in milliseconds
   */
  getDelta(now: number) {
    const delta = (now - this.lastTick) / 1000;
    this.lastTick = now;
    return delta;
  }
}
